class domain {
 private:
  int size_x, size_y, size_z;
  int location_x, location_y, location_z;
  double *vertex;
  int loc(int,int,int);
  int pos_x(int);
  int pos_y(int);
  int pos_z(int);
  void positions(int,int&,int&,int&);
  double relValue(int,int,int);
 public:
  domain ();
  domain (int);
  domain (int,int,int);

  int size (void);
  int sizeX (void);
  int sizeY (void);
  int sizeZ (void);

  void wright (void);

  void setLocation (int,int,int);
  void setValue (int,int,int,double);
  void evolve(double,double);
  void evolve(double,double,domain*);
  void emit(double,double,int);
//void emit(double,double,int,domain*);
//void transmit(double,double,domain*);
  double value(int);
  double value(int,int,int);

  ~domain ();
 };

 domain::domain () {
  };

 domain::domain (int x) {
  size_x=x;
  size_y=x;
  size_z=x;
  vertex = new double [this->size()];
  };

 domain::domain (int x, int y, int z) {
  size_x=x;
  size_y=y;
  size_z=z;
  vertex = new double [this->size()];
  };

 int domain::loc(int x, int y, int z){
  return ((z-1)*size_x*size_y+(y-1)*size_x+x);
  };

 int domain::pos_x(int q){
  int a=this->size();
  return 1+q-a*(this->pos_y(q)-1)-a*a*(this->pos_z(q)-1);
  };


 int domain::pos_y(int q){
  int a=this->size();
  return 1+q/a-a*(this->pos_z(q)-1);
  };

 int domain::pos_z(int q){
  int a=this->size();
  return q/(a*a)+1;
  };

 void domain::positions (int q, int& x, int& y, int& z) {
  int a=this->size();
  z=q/(a*a);
  y=q/a-a*(z);
  x=q-a*(y)-a*a*(z);
  z+=location_z;
  y+=location_y;
  x+=location_x;
  };

 int domain::size(void) {
  return (size_x*size_y*size_z);
  };

 void domain::setLocation(int x, int y, int z) {
  location_x=x;
  location_y=y;
  location_z=z;
  };

 void domain::setValue (int x, int y, int z, double v){
  int k=(z-1)*size_x*size_y+(y-1)*size_x+x;
  vertex[this->loc(x,y,z)]=v;
  };

 void domain::evolve(double d, double c){
  std::cout << "d=" << d << std::endl;
  for (int i;i<=this->size();i++){
   vertex[i]+=d*vertex[i]+c;
   };
  };

 void domain::evolve(double d, double c, domain * m){
  int x,y,z;
  for (int i;i<=this->size();i++){
   this->positions(i,x,y,z);
   vertex[i]=(1+d)*m->value(x,y,z)+c;
   };
  };

 void domain::emit(double d, double c, int r){
  //pocet bodov pre ktore sa bude robit zmena
  //c = extinkcny koeficient
  //d = inkrement zavisly od vlastnej koncentracie
  //r = maximalny dosah interakcie
  int a=2*r+1;
  int x,y,z,a3,q;
  int x0,y0,z0,min_x,min_y,min_z;
  int max_x,max_y,max_z,dx,dy,dz,l;
  double * virt = new double [this->size()];
  for (int i=1;i<=this->size();i++) {
    z0=i/(a*a)+1;
    min_z=z0-r;
    if (min_z<1) min_z=1;
    max_z=z0+r;
    if (max_z>size_z) max_z=size_z;
    y0=1+i/a-a*(z0-1);
    min_y=y0-r;
    if (min_y<1) min_y=1;
    max_y=y0+r;
    if (max_y>size_y) max_y=size_y;
    x0=1+i-a*(y0-1)-a*a*(z0-1);
    min_x=x0-r;
    if (min_x<1) min_x=1;
    max_x=x0+r;
    if (max_x>size_x) max_x=size_x;
    dx=(max_x-min_x);
    dy=(max_y-min_y);
    dz=(max_z-min_z);
    a3=dx*dy*dz;
   for (int j=1;j<=a3;j++){
    //q je pozicia v mensej kocke vyratana z j
    z=min_z+j/(dx*dy);
    y=min_y+j/dx-dy*(z-1);
    x=min_x+j-dy*(y-1)-dx*dy*(z0-1);
    q=this->loc(x,y,z);
    l=(x0-x)*(x0-x)+(y0-y)*(y0-y)+(z0-z)*(z0-z);
    l=sqrt(l);
    virt[q]=vertex[q]-d*exp(c)*vertex[i]/(4*M_PI*l*l);
    };
   };
   //update vertexu a vymazanie virtu
  for (int i=0;i<=this->size();i++) {
  vertex[i]=virt[i];
  };
  delete [] virt;
  };

// void domain::huygens(double c, int t){
//  };

 double domain::relValue(int x, int y, int z){
  return (vertex[this->loc(x,y,z)]);
  };

 double domain::value(int x, int y, int z) {
  return (vertex[this->loc(x-location_x+1,y-location_y+1,z-location_z+1)]);
  };

 double domain::value(int x) {
  return (vertex[x]);
  };

 int domain::sizeX (void) {
  return (size_x);
  };
 
 int domain::sizeY (void) {
  return (size_y);
  };

 int domain::sizeZ (void) {
  return (size_z);
  };

 void domain::wright (void) {
  for (int i=1;i<=size_z;i++) {
   for (int j=1;j<=size_y;j++) {
    for (int k=1;k<=size_x;k++) {
     std::cout << this->relValue(k,j,i) << " ";
     };
    std::cout << std::endl;
    };
   std::cout << std::endl;
   };
  };

domain::~domain () {
 delete [] vertex;
 };
